/******************************************************************************
 * PlayerReportPane.java - 
 * $Id: PlayerReportPane.java,v 1.3 2010/01/23 06:16:41 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright 2009 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: PlayerReportPane.java,v $
 * Revision 1.3  2010/01/23 06:16:41  dick
 * Better RepBot parsing.
 *
 * Revision 1.2  2009/03/12 15:28:43  dick
 * PlayerReportPane looks good.
 *
 * Revision 1.1  2009/03/04 19:05:24  dick
 * Display the panel with what we know about a player.
 *
 */

/* 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * The Original Code is BuckoFIBS, <http://www.buckosoft.com/BuckoFIBS/>.
 * The Initial Developer of the Original Code is Dick Balaska and BuckoSoft, Corp.
 * 
 */
package com.buckosoft.fibs.BuckoFIBS.gui;

import java.text.SimpleDateFormat;
import java.util.List;

import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;

import com.buckosoft.fibs.BuckoFIBS.CommandDispatcher;
import com.buckosoft.fibs.BuckoFIBS.db.Database;
import com.buckosoft.fibs.domain.FinishedMatch;
import com.buckosoft.fibs.domain.Player;
import com.buckosoft.fibs.net.FIBSMessages;

/** Display the panel with what we know about a player.
 * @author Dick Balaska
 * @since Mar 2, 2009
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/BuckoFIBS/gui/PlayerReportPane.java">cvs PlayerReportPane.java</a>
 */
public class PlayerReportPane extends JScrollPane implements FIBSMessages {
	private	final static boolean DEBUG = true;
	private static final long serialVersionUID = 1L;
	private	CommandDispatcher commandDispatcher;  //  @jve:decl-index=0:
	private	Database			db;
	
	private	JTree		tree = null;
	private	DefaultTreeModel	treeModel = null;
	private Node		topNode = new Node("Player:");  //  @jve:decl-index=0:
	private	Node		repbotNode = null;
	private Node		whoisNode = null;
	private	Node		recordNode = null;
	private Node		savedNode = null;
	private	TreePath	pathToRepbot = null;
	private	TreePath	pathToWhois = null;
	private	TreePath	pathToRecord = null;
	
	SimpleDateFormat	sdf = new SimpleDateFormat("dd MMM yy - HH:mm");

	private	String	playerName;

	private	String[]	commands = {
		"whois",
		"show savedcount",
		"Tell repbot ask",
		"Tell repbot list"
	};
	private	int			nextCommand = 0;

	/** Create a PlayerReportPane
	 */
	public PlayerReportPane() {
		super();
		
		initialize();
	}

	/** I just don't want to type that long name... 
	 * If i could type #define Node DefaultMutableTreeNode, i would.
	 */
	class Node extends DefaultMutableTreeNode {
		private static final long serialVersionUID = 1L;
		Node() { super(); }
		Node(Object o) { super(o); }
	}

	/** Set the reference to the command dispatcher
	 * @param commandDispatcher Our system CommandDispatcher
	 */
	public	void	setCommandDispatcher(CommandDispatcher commandDispatcher) {
		this.commandDispatcher = commandDispatcher;
	}

	public void setDatabase(Database database) {
		this.db = database;
	}

	/** Set the Player that we will be looking up.
	 * @param playerName
	 */
	public void setPlayer(String playerName) {
		this.playerName = playerName;
		topNode.setUserObject("Player: " + playerName);
		topNode.removeAllChildren();
		this.repbotNode.removeAllChildren();
		this.whoisNode.removeAllChildren();
		this.recordNode = null;
		pathToRecord = null;
		createNodes(topNode);
		updateGui();
		nextCommand = 0;
		sendNextCommand();
		parseRecord();
	}

	/** Receive a line from FIBS.
	 * @param s
	 * @param cookie
	 */
	public void receiveLine(String s, int cookie) {
		System.out.println("PlayerReport..line " + s);
		
		if (s.startsWith("12 RepBotNG "))
			parseRepbot(s);
		else if (cookie == FIBS_HasSavedGames || cookie == FIBS_HasNoSavedGames)
			parseSavedGames(s);
		else
			parseWhois(s, cookie);
	}

	private void parseSavedGames(String s) {
		int i = s.indexOf(' ');
		s = s.substring(i+4);
		if (savedNode == null)
			savedNode = new Node(s);
		else
			savedNode.setUserObject(s);
		topNode.add(savedNode);
		sendNextCommand();
	}

	private void parseRepbot(String s) {
		System.out.println("parseRepbot '" + s + "'");
		s = s.substring(12);		// Scrape off '12 RepBotNG '
		if (s.startsWith("You have")) {
			this.commandDispatcher.writeSystemMessageln(SystemMessagesTextPane.ERROR,
					"RepBot says: " + s);
			return;
		}
		int i = s.indexOf('\''); 	// "Player's vouchers..."
		if (i != -1) {
			String name = s.substring(0, i);
			if (!name.equals(this.playerName)) {
				String t = "Repbot.. NOT player: '" + s + "'";
				if (DEBUG)
					System.out.println(t);
				this.commandDispatcher.writeSystemMessageln(t);
				return;
			}
			s = s.substring(i+3);
			if (s.startsWith("reputation")) {
				parseRepbotReputation(s);
				sendNextCommand();
			} else
				parseRepbotOther(s);
			return;
		}
		if (s.startsWith("User ")) {
			parseRepbotReputation(s);
			return;
		}
		if (this.playerName != null && s.startsWith(this.playerName)) {
			s = s.substring(this.playerName.length() + 1);
			parseRepbotOther(s);
			return;
		}
		if (DEBUG)
			System.out.println("Repbot.. busted line: '" + s + "'");
		this.commandDispatcher.writeSystemMessageln("Repbot: " + s);	
	}

	private void parseRepbotReputation(String s) {
		repbotNode.setUserObject("RepBot " + s);
		updateGui();
	}

	private void parseRepbotOther(String s) {
		if (DEBUG)
			System.out.println("Repot other: = '" + s + "'");
		repbotNode.add(new Node(s));
		updateGui();
		
	}

	private void parseWhois(String s, int cookie) {
		if (cookie == FIBS_PlayerInfoStart)
			whoisNode.setUserObject("whois " + s);
		else
			whoisNode.add(new Node(s));
		updateGui();
		if (cookie == FIBS_NoEmail || cookie == FIBS_EmailAddress)
			sendNextCommand();
	}

	/** Not really a parse, because we know all the info
	 */
	private void parseRecord() {
		Player p = this.db.getPlayer(this.playerName);
		if (p == null)
			return;
		List<FinishedMatch> matches = db.getFinishedMatches(p.getId());
		if (matches.size() == 0)
			return;
		if (recordNode == null)
			recordNode = new Node();
		else
			recordNode.removeAllChildren();
		int w = 0;
		int l = 0;
		for (FinishedMatch fm : matches) {
			StringBuffer sb = new StringBuffer();
			if (fm.getYourScore() > fm.getOpponentScore())
				w++;
			else
				l++;
			sb.append("" + fm.getYourScore());
			sb.append("-");
			sb.append("" + fm.getOpponentScore());
			//sb.append("  1-2,  13 Aug 03 02:34");
			sb.append(";");
			sb.append("" + fm.getMatchPoints());
			sb.append(",  ");
			sb.append(sdf.format(fm.getDate()));
			int d = fm.getDuration();
			int m = d/60;
			int s = d-m*60;
			sb.append("  t=" + m + ":" + s);
			//sb.append("  --" + d);
			
			recordNode.add(new Node(sb.toString()));
		}
		recordNode.setUserObject("" + w + "-" + l);
		this.topNode.add(recordNode);
		this.pathToRecord = new TreePath(treeModel.getPathToRoot(this.recordNode));
	}

	private void sendNextCommand() {
		if (nextCommand >= commands.length)
			return;
		String s = commands[nextCommand++] + " " + this.playerName;
		this.commandDispatcher.dispatch(CommandDispatcher.Command.SEND_COMMAND, s);
	}

	private void updateGui() {
		treeModel.reload();
		if (pathToRecord != null)
			tree.expandPath(pathToRecord);
		tree.expandPath(pathToRepbot);
		tree.expandPath(pathToWhois);
	}

	/**
	 * This method initializes this
	 * 
	 */
	private void initialize() {
        this.setViewportView(getTree());
	}

	/**
	 * This method initializes jTree	
	 * 	
	 * @return javax.swing.JTree	
	 */
	private JTree getTree() {
		if (tree == null) {
			treeModel = new DefaultTreeModel(topNode);
			createNodes(topNode);
			tree = new JTree(treeModel);
			tree.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
		}
		return tree;
	}
	
	private void createNodes(DefaultMutableTreeNode top) {
		//if (repbotNode == null)
		repbotNode = new Node("RepBot:"); 
		top.add(repbotNode);
		if (whoisNode == null)
			whoisNode = new Node("whois:"); 
		top.add(whoisNode);
		TreeNode[] tn = treeModel.getPathToRoot(this.repbotNode);
		pathToRepbot = new TreePath(tn);
		this.pathToWhois = new TreePath(treeModel.getPathToRoot(this.whoisNode));
	
	}
}
