/******************************************************************************
 * ConnectToServerDialog.java - Manage the Connect to Server dialog
 * $Id: ConnectToServerDialog.java,v 1.8 2009/01/28 18:15:10 dick Exp $
 * 
 * BuckoFIBS - Backgammon by BuckoSoft
 * Copyright 2008 - Dick Balaska - BuckoSoft, Corp.
 * 
 * $Log: ConnectToServerDialog.java,v $
 * Revision 1.8  2009/01/28 18:15:10  dick
 * Prettier cvs link in the javadoc.
 *
 * Revision 1.7  2009/01/18 05:00:40  dick
 * The owner is a JFrame, not a JDialog.
 *
 * Revision 1.6  2009/01/09 07:17:38  dick
 * Initialize the data after constructing the widget.
 *
 * Revision 1.5  2009/01/07 20:03:22  dick
 * Cleanups and Javadoc.
 *
 * Revision 1.4  2009/01/07 08:33:31  dick
 * Change the settings when the profile changes.
 *
 * Revision 1.3  2009/01/06 08:10:37  dick
 * Don't use deprecated password stuff.
 *
 * Revision 1.2  2009/01/05 07:16:09  dick
 * @see moved to .../account
 *
 * Revision 1.1  2009/01/05 07:09:53  dick
 * ConnectToServerDialog and CreateAccountDialog moved to account.
 *
 * Revision 1.5  2009/01/04 19:57:49  dick
 * Add the New Account handling.
 *
 * Revision 1.4  2008/12/11 08:47:55  dick
 * Center the dialog over the MainDialog. (Looks like poo, but better than being at 0,0).
 *
 * Revision 1.3  2008/10/03 06:57:08  dick
 * Set modal to true.
 *
 * Revision 1.2  2008/03/31 07:10:11  dick
 * Add the username and password.
 * Use a GridBagLayout.
 *
 * Revision 1.1  2008/03/30 05:42:26  dick
 * Make sure we are connecting correctly...
 *
 */
package com.buckosoft.fibs.BuckoFIBS.gui.account;

import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JPasswordField;
import javax.swing.JTextField;

import com.buckosoft.fibs.BuckoFIBS.BFProperties;
import com.buckosoft.fibs.BuckoFIBS.CommandDispatcher;

/** Manage the Connect to Server dialog.
 * @author Dick Balaska
 * @since 2008/03/30
 * @version $Revision: 1.8 $ <br> $Date: 2009/01/28 18:15:10 $
 * @see <a href="http://cvs.buckosoft.com/Projects/BuckoFIBS/BuckoFIBS/src/com/buckosoft/fibs/BuckoFIBS/gui/account/ConnectToServerDialog.java">cvs ConnectToServerDialog.java</a>
 */
public class ConnectToServerDialog extends JDialog implements ProfileSelectorNotifier {

	private static final long serialVersionUID = 1L;
	private JPanel jContentPane = null;
	private JLabel serverNameLabel = null;
	private JButton jButtonConnect = null;
	private JButton jButtonCancel = null;
	private	CommandDispatcher	commandDispatcher;  //  @jve:decl-index=0:
	private	BFProperties		properties;
	private JLabel userNameLabel = null;
	private JTextField userNameField = null;
	private JLabel passwordLabel = null;
	private JPasswordField passwordTextField = null;
	private JButton jCreateNewAccountButton = null;
	private JPanel jButtonPanel = null;
	private ServerPortSelectorPanel jPanelServerPort = null;
	private JLabel jLabelProfiles = null;
	private ProfileSelectorPanel jPanelProfileSelector = null;
	
	/** Create the Connect to Server dialog
	 * @param owner The parent dialog
	 * @param commandDispatcher The commandDispatcher
	 */
	public ConnectToServerDialog(JFrame owner, CommandDispatcher commandDispatcher) {
		super(owner, true);
		this.commandDispatcher = commandDispatcher;
		this.properties = commandDispatcher.getProperties();
		initialize();
		this.setLocationRelativeTo(owner);
		activateProfileObjects();
		this.profileChanged();
	}

	private void activateProfileObjects() {
		boolean b = this.properties.isAllowMultiplePersonalities();
		this.jLabelProfiles.setVisible(b);
		this.jPanelProfileSelector.setVisible(b);
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
		this.setSize(309, 211);
		this.setModal(true);
		this.setTitle("Connect To Server");
		this.setContentPane(getJContentPane());
		userNameField.setText(this.properties.getUserName(jPanelProfileSelector.getSelectedProfile()));
		passwordTextField.setText(this.properties.getPassword(jPanelProfileSelector.getSelectedProfile()));
	}

	private void connectButtonActionPerformed(java.awt.event.ActionEvent evt) {
		setVisible(false);
		int profileId = jPanelProfileSelector.getSelectedProfile();
		this.properties.setUserName(profileId, this.userNameField.getText());
		this.properties.setPassword(profileId, new String(this.passwordTextField.getPassword()));
		this.properties.setServerName(profileId, (String)this.jPanelServerPort.getServerName());
		this.properties.setServerPort(profileId, this.jPanelServerPort.getPort());
		dispose();
		commandDispatcher.dispatch(CommandDispatcher.Command.CONNECT_TO_SERVER);
	}

	private void cancelButtonActionPerformed(java.awt.event.ActionEvent evt) {
		setVisible(false);
		dispose();
	}

	private void newAccountButtonActionPerformed(java.awt.event.ActionEvent evt) {
		setVisible(false);
		dispose();
		commandDispatcher.dispatch(CommandDispatcher.Command.SHOW_NEW_ACCOUNT_DIALOG);
	}

	
	/* (non-Javadoc)
	 * @see com.buckosoft.fibs.BuckoFIBS.gui.account.ProfileSelectorNotifier#profileChanged()
	 */
	@Override
	public void profileChanged() {
		int profileId = this.properties.getSelectedProfile();
		this.userNameField.setText(this.properties.getUserName(profileId));
		this.passwordTextField.setText(this.properties.getPassword(profileId));
		this.jPanelServerPort.setServerName(this.properties.getServerName(profileId));
	}

	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private JPanel getJContentPane() {
		if (jContentPane == null) {
			GridBagConstraints gridBagConstraints71 = new GridBagConstraints();
			gridBagConstraints71.gridx = 1;
			gridBagConstraints71.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints71.gridy = 0;
			GridBagConstraints gridBagConstraints61 = new GridBagConstraints();
			gridBagConstraints61.gridx = 0;
			gridBagConstraints61.anchor = GridBagConstraints.EAST;
			gridBagConstraints61.gridy = 0;
			jLabelProfiles = new JLabel();
			jLabelProfiles.setText("Profile:");
			GridBagConstraints gridBagConstraints51 = new GridBagConstraints();
			gridBagConstraints51.gridx = 1;
			gridBagConstraints51.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints51.weightx = 1.0;
			gridBagConstraints51.anchor = GridBagConstraints.CENTER;
			gridBagConstraints51.gridy = 1;
			GridBagConstraints gridBagConstraints31 = new GridBagConstraints();
			gridBagConstraints31.gridx = 0;
			gridBagConstraints31.gridwidth = 2;
			gridBagConstraints31.insets = new Insets(0, 0, 5, 0);
			gridBagConstraints31.anchor = GridBagConstraints.SOUTH;
			gridBagConstraints31.weighty = 1.0;
			gridBagConstraints31.gridy = 10;
			GridBagConstraints gridBagConstraints9 = new GridBagConstraints();
			gridBagConstraints9.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints9.gridy = 4;
			gridBagConstraints9.weightx = 1.0;
			gridBagConstraints9.insets = new Insets(0, 0, 30, 0);
			gridBagConstraints9.gridwidth = 1;
			gridBagConstraints9.gridx = 1;
			GridBagConstraints gridBagConstraints8 = new GridBagConstraints();
			gridBagConstraints8.gridx = 0;
			gridBagConstraints8.anchor = GridBagConstraints.WEST;
			gridBagConstraints8.ipady = 0;
			gridBagConstraints8.insets = new Insets(0, 0, 30, 0);
			gridBagConstraints8.gridy = 4;
			passwordLabel = new JLabel();
			passwordLabel.setText("Password:");
			GridBagConstraints gridBagConstraints7 = new GridBagConstraints();
			gridBagConstraints7.fill = GridBagConstraints.HORIZONTAL;
			gridBagConstraints7.gridy = 3;
			gridBagConstraints7.weightx = 1.0;
			gridBagConstraints7.gridwidth = 1;
			gridBagConstraints7.gridx = 1;
			GridBagConstraints gridBagConstraints6 = new GridBagConstraints();
			gridBagConstraints6.gridx = 0;
			gridBagConstraints6.anchor = GridBagConstraints.WEST;
			gridBagConstraints6.gridy = 3;
			userNameLabel = new JLabel();
			userNameLabel.setText("Username:");
			GridBagConstraints gridBagConstraints = new GridBagConstraints();
			gridBagConstraints.gridx = 0;
			gridBagConstraints.ipadx = 0;
			gridBagConstraints.ipady = 4;
			gridBagConstraints.anchor = GridBagConstraints.EAST;
			gridBagConstraints.gridy = 1;
			serverNameLabel = new JLabel();
			serverNameLabel.setText("Server:");
			jContentPane = new JPanel();
			jContentPane.setLayout(new GridBagLayout());
			jContentPane.add(serverNameLabel, gridBagConstraints);
			jContentPane.add(userNameLabel, gridBagConstraints6);
			jContentPane.add(getUserNameField(), gridBagConstraints7);
			jContentPane.add(passwordLabel, gridBagConstraints8);
			jContentPane.add(getPasswordTextField(), gridBagConstraints9);
			jContentPane.add(getJButtonPanel(), gridBagConstraints31);
			jContentPane.add(getJPanelServerPort(), gridBagConstraints51);
			jContentPane.add(jLabelProfiles, gridBagConstraints61);
			jContentPane.add(getJPanelProfileSelector(), gridBagConstraints71);
		}
		return jContentPane;
	}

	/** This method initializes userNameField	
	 * @return javax.swing.JTextField	
	 */
	private JTextField getUserNameField() {
		if (userNameField == null) {
			userNameField = new JTextField();
		}
		return userNameField;
	}

	/**
	 * This method initializes passwordTextField	
	 * 	
	 * @return javax.swing.JTextField	
	 */
	private JPasswordField getPasswordTextField() {
		if (passwordTextField == null) {
			passwordTextField = new JPasswordField();
		}
		return passwordTextField;
	}

	/**
	 * This method initializes jButtonConnect	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonConnect() {
		if (jButtonConnect == null) {
			jButtonConnect = new JButton();
			jButtonConnect.setText("Connect");
			jButtonConnect.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					connectButtonActionPerformed(e);
				}
			});
		}
		return jButtonConnect;
	}

	/**
	 * This method initializes jButtonCancel	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJButtonCancel() {
		if (jButtonCancel == null) {
			jButtonCancel = new JButton();
			jButtonCancel.setText("Cancel");
			jButtonCancel.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					cancelButtonActionPerformed(e);
				}
			});
		}
		return jButtonCancel;
	}

	/**
	 * This method initializes jCreateNewAccountButton	
	 * 	
	 * @return javax.swing.JButton	
	 */
	private JButton getJCreateNewAccountButton() {
		if (jCreateNewAccountButton == null) {
			jCreateNewAccountButton = new JButton();
			jCreateNewAccountButton.setText("New Account");
			jCreateNewAccountButton.addActionListener(new java.awt.event.ActionListener() {
				public void actionPerformed(java.awt.event.ActionEvent e) {
					newAccountButtonActionPerformed(e);
				}
			});
		}
		return jCreateNewAccountButton;
	}

	/**
	 * This method initializes jButtonPanel	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private JPanel getJButtonPanel() {
		if (jButtonPanel == null) {
			GridBagConstraints gridBagConstraints5 = new GridBagConstraints();
			gridBagConstraints5.insets = new Insets(0, 4, 1, 4);
			gridBagConstraints5.gridy = -1;
			gridBagConstraints5.weightx = 0.3;
			gridBagConstraints5.gridx = -1;
			GridBagConstraints gridBagConstraints4 = new GridBagConstraints();
			gridBagConstraints4.insets = new Insets(0, 4, 1, 0);
			gridBagConstraints4.gridy = -1;
			gridBagConstraints4.ipadx = 0;
			gridBagConstraints4.ipady = 0;
			gridBagConstraints4.weightx = 0.3;
			gridBagConstraints4.gridx = -1;
			GridBagConstraints gridBagConstraints1 = new GridBagConstraints();
			gridBagConstraints1.insets = new Insets(0, 0, 1, 4);
			gridBagConstraints1.gridy = -1;
			gridBagConstraints1.ipadx = 0;
			gridBagConstraints1.ipady = 0;
			gridBagConstraints1.weightx = 0.3;
			gridBagConstraints1.gridx = -1;
			jButtonPanel = new JPanel();
			jButtonPanel.setLayout(new GridBagLayout());
			jButtonPanel.add(getJButtonConnect(), gridBagConstraints1);
			jButtonPanel.add(getJCreateNewAccountButton(), gridBagConstraints5);
			jButtonPanel.add(getJButtonCancel(), gridBagConstraints4);
		}
		return jButtonPanel;
	}

	/**
	 * This method initializes jPanelServerPort	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private ServerPortSelectorPanel getJPanelServerPort() {
		if (jPanelServerPort == null) {
			jPanelServerPort = new ServerPortSelectorPanel();
			jPanelServerPort.setLayout(new GridBagLayout());
		}
		return jPanelServerPort;
	}

	/**
	 * This method initializes jPanelProfileSelector	
	 * 	
	 * @return javax.swing.JPanel	
	 */
	private ProfileSelectorPanel getJPanelProfileSelector() {
		if (jPanelProfileSelector == null) {
			jPanelProfileSelector = new ProfileSelectorPanel(this, this, properties);
			jPanelProfileSelector.setLayout(new GridBagLayout());
		}
		return jPanelProfileSelector;
	}

}  //  @jve:decl-index=0:visual-constraint="10,10"
